function [Y,P] = knuth(p,deb)
% Genera le lookup tables per estrarre 
% 1,2,...,N con probabilita` p1,p2,...,pN
% secondo l'alg. di Knuth
%
% Usage: [Y,P] = knuth(p,deb)
% p=(p1,p2,...,pN)
% Se deb>0 si effettua il check della tabella
% 
%! L`algoritmo risolve un analogo problema di logistica:
%! Dati n1 pacchi uguali da recapitare all`indirizzo I1,
%! n2 per I2, etc., fino a N indirizzi, si tratta di distribuire
%! i pacchi a N fattorini ognuno dei quali puo` portare esattamente
%! sum(nk)/N pacchi e non vuole visitare piu` di due indirizzi.

[N,M]=size(p);
if N<M,
	p=p';
	N=M;
end

[L,ind]=sort(p);
L=[L ind];
n=N;

while n>1,
  P(L(1,2))=N*L(1,1);
  Y(L(1,2))=L(n,2);
  L(n,1)=L(n,1)-(1/N-L(1,1));
  L=L(2:n,:);
  [tmp,ind]=sort(L(:,1));
  L=L(ind,:);
  n=n-1;
end % now n=1, ... last entry

P(L(1,2))=N*L(1,1);
Y(L(1,2))=L(n,2);


if nargin>1, % debug 
  for j=1:N,
    P1=1-P(find(Y==j));
    q=[q; (P(j)+sum(P1))/N];
  end
  error=	norm(q-p)/N;		
  if(error<=2*eps),
    disp('La tabella e` corretta a livello di precisione di macchina');
  else
    disp('La tabella contiene un errore:');
    error
  end
end


