function Kdemo(Nevents)
% application of Knuth's alias method 
% Source: D.Knuth, The Art of Computer Programming, Vol.2
% See also: Computing in Science and Eng., July issue 1999, paper
% by I.Beichl and F.Sullivan
%
% E.Onofri  2000-2018

if nargin<1,
  Nevents=1e5;
elseif(Nevents>1e6)                     % do not overcharge your computer!
  Nevents=1e6;
end

close all

check='go';

while(check=='go'),
  selection = {'dice throw','diffraction pattern','poisson', ...
	       'gaussian','lorentzian','linear','quit'};
  choice    = menu('Start',selection);
	
  switch choice		
    case 1					% Knuth's example
      result=zeros(3,1);
      disp('Alice, Bruno and Charles are drawing dice for 100 times')
      players={'Alice';'Bruno';'Charles'};
      p=[1 2 3 4 5 6 5 4 3 2 1];		
      KLT=kalias(p);

      for j=1:100,
         X = Krand(3,KLT);
         s = max(X);
         ind = find(X==s);
         result(ind) = result(ind) + 1/length(ind);
      end
      score=max(result);
      win=find(result==score);
      disp(['The winner is ' players{win} ' by ' num2str(score) ' points ']);
      hold off
      bar(result);
    case 2			% fringe
      disp('Running <diffraction pattern demo>');
      disp([num2str(Nevents)...
            ' scintillation counts from low energy electron beam']);      
      x = linspace(0,1,128);
      tic;KLTc = krndini('fringe',x,4,0);toc
      X=Krandc(Nevents,KLTc);
      figure
      plot(X,rand(size(X)),'k.','Markersize',1)
      legend('Diffraction pattern')
      
    case 3					% Poisson
      disp(['Extracting ' num2str(Nevents) ' events from Poisson distribution']);
      
      p = poisson(64,10);
      n = 0:64;
      KLT = kalias(p);
      tic;X = Krand(Nevents,KLT)-1;toc
      histogram(X,64,'BinMethod','fd',... % binning Freedman-Diaconis
                'DisplayStyle','stairs',...
                'FaceAlpha',.6,...%             'FaceColor','none',...
                'EdgeColor','auto',...
                'Normalization','probability');    % normalization=prob.distr.fnct);
      
      grid
      hold on
      
      plot(n,p)
      ax=axis;
      axis([0 max(X) ax(3) ax(4)]);
      disp(['<X>=' num2str(mean(X)) ' +/- ' num2str(std(X))])
      toc
   case 4					% gaussian
    disp(['Extracting ' num2str(Nevents)...
	  ' events from Gaussian distribution']);
    x=linspace(-10,10,200);
    tic;KLTc=krndini('gaussian',x,0,2);toc
    X=Krandc(Nevents,KLTc);
   case 5
    disp(['Extracting ' num2str(Nevents)...
	  ' events from Lorentzian distribution']);
    x=linspace(-100,100,400);		% Lorentzian
    tic;KLTc=krndini('lorentz',x,0,20);toc
    X=Krandc(Nevents,KLTc);
   case 6
    disp(['Extracting ' num2str(Nevents)...
	  ' events from linear distribution']);
    n=1:100;
    p=n/sum(n);
    tic;KLT=kalias(p);
    X=Krand(Nevents,KLT);t1=toc;
    tic;exact=lpdf(Nevents,100);t2=toc;
    histogram(X,50,'Normalization','pdf','DisplayStyle','stairs');
    hold on
    histogram(exact,50,'Normalization','pdf','DisplayStyle','stairs');
    title('freq(n)/p(n) ')
    grid
    % axis([0,100,.8,1.2])
    disp(['Time for alias method:',num2str(t1)])
    disp(['Time for direct method:',num2str(t2)])
    
   otherwise
    check='quit';
    return
  end
end
end
%--------
function   showhist(x,prob,X);
   hold off
   n=length(x);
   histogram(X,n/4,'BinMethod','fd',... % binning Freedman-Diaconis
             'DisplayStyle','bar',...
             'FaceAlpha',.4,...
             'FaceColor','cyan',...
             'EdgeColor','auto',...
             'EdgeAlpha',1,...
             'Normalization','pdf');    % normalization=prob.distr.fnct

             
   hold on
   dx=x(2)-x(1);
   NRAND=length(X);
   plot(x,prob/dx,'r','LineWidth',1.5);   % normalization: sum(p*dx)=1
   hold off
   grid
   legend('random sample','probability distribution');
end

