function X = Krand(NRAND,KLT,seed) 
% Returns NRAND random integers in the interval 1:n (n = length(Y))
% distributed according to the probability vector p(1) p(2) ... p(n)
% which is encoded in the lookup tables KLT.Y and KLT.P prepared through
% Knuth's alias method: KLT=kalias(p); 
% Usage:
% X=krand(N,KLT,'init') % returns N random elements from the
% set (1...length(Y)) after initializing the random number generator,
% seed is chosen as 'sum(clock)*100
%
% X=krand(N,KLT,number) % returns N random elements from the
% set (1...length(Y)) after initializing the random number generator,
% using seed=number
%
% X=krand(1e6,KLT) % returns one million random elements from the
% set (1...length(Y)) without resetting rng's seed
%
% Requires: kalias.m
% Source: D.Knuth, The Art of Computer Programming, Vol.2
% See also: Computing in Science and Eng., July issue 1999, paper
% by I.Beichl and F.Sullivan
% E.Onofri, 1998-2002

if nargin==3,
   if isstr(seed),
      rand('state',sum(clock)*100);
   else
      rand('state',seed);
   end
end

if nargin>1,
   P = KLT.P;
   Y = KLT.Y;
   N = length(P);
   v = rand(1,NRAND);
   X = randi(N,NRAND,1);
   
   ind = find(v>P(X));
   X(ind) = Y(X(ind)); 
else
   disp('please provide all input')
   disp('Usage: X=Krand(N,K)')
   disp('returns N sample data extracted according to table K')
   disp('K should be built using Kalias')
end

% previously:
   % r = N*rand(1,NRAND);  
   % X = ceil(r);
   % v = X-r;
