function X=krandc(NRAND,x,prob,Y,P,seed,plot)
% Returns NRAND random numbers distributed
% according to the density prob(x)
% defined on the grid [x1,x2,...,xn] 
% and linearly interpolated to cover the closed interval [x1,xn].
% Usage: X=krandc(NRAND,x,prob,Y,P [,seed] [,plot])
% Y,P are provided by krndini.m
% seed is optional: it can be an integer or a string ('auto'),
% in this latter case seed is chosen by reading the system clock.
% if plot is non-nil a histogram will be plotted
% For discrete distributions use krand.m

if nargin==6,
  if isstr(seed),
    rand('seed',sum(clock)*100);
  else
    rand('seed',seed);
  end
end

if(length(x)==length(P)+1),
  disp('');
else
  disp('Error: check dimensions');
  disp('Y and P must be n-1 dim if dim(x)=n');
end

N=length(P);

tmp=zeros(1,NRAND);
r=N*rand(1,NRAND);  
j=floor(r);

v=r-j; % Are low digits random enough?
%v=rand(1,NRAND); % if not use this line instead

tmp=j+1;
ind=find(v>P(j+1));
tmp(ind)=Y(j(ind)+1);
X0=x(tmp); % coarse grained distribution

n=length(x);
right=2:n; 
left=right-1;
a=x(right)-x(left);  % Do not assume equally spaced grid for later use

p=(prob(right)+prob(left));
delta=(prob(right)-prob(left))./p; % linearized prob distribution

a=a(tmp);
delta=delta(tmp);

r=rand(1,NRAND);

csi=2*r./(1-delta + sqrt((1-delta).^2+4*delta.*r)); % csi gives a random
                                   % variable in the coarse grained bins 
X=X0+csi.*a;

if nargin==7,
  showhist(x,prob,X);
end
