function kdemo(Nevents)
% application of Knuth's alias method 
% Source: D.Knuth, The Art of Computer Programming, Vol.2
% See also: Computing in Science and Eng., July issue 1999, paper
% by I.Beichl and F.Sullivan
%
% E.Onofri 2000
% Right to copy for the whole package
% 10.00 - to be accredited to the
% "Physics Promotion Fund", c/o Dipartimento di Fisica, Universit di Parma
% Parco Area delle Scienze, 43100 Parma

if nargin<1,
  Nevents=1e5;
%elseif(Nevents>1e6)
%  Nevents=1e6;
end

close all

check='go';

while(check=='go'),
  selection = {'dice throw','diffraction pattern','poisson', ...
	       'gaussian','lorentzian','linear','quit'};
  choice    = menu('Start',selection);
	
  switch choice		
   case 1					% Knuth's example
    result=zeros(3,1);
    disp('Alice, Bruno and Charles are drawing dice for 100 times')
    players={'Alice';'Bruno';'Charles'};
    p=[1 2 3 4 5 6 5 4 3 2 1];		
    KLT=kalias(p)
    krand(1,KLT,'init');
    for j=1:100,
      X=krand(3,KLT);
      s=max(X);
      ind=find(X==s);
      result(ind)=result(ind)+1/length(ind);
    end
    score=max(result);
    win=find(result==score);
    disp(['The winner is ' players{win} ' by ' num2str(score) ' points ']);
    bar(result);
   case 2			% fringe
    disp('Running <diffraction pattern demo>');
    disp([num2str(Nevents)...
	  ' scintillation counts from low energy electron beam']);
    
    x=linspace(0,1,100);
    tic;KLTc=krndini('fringe',x,4,0);toc
    X=krandc(Nevents,KLTc);
    figure
    plot(X,rand(size(X)),'b.','Markersize',1)
    legend('Diffraction pattern')
    
   case 3					% Poisson
    disp(['Extracting ' num2str(Nevents) ' events from Poisson distribution']);
    
    p=poisson(50,10);
    KLT=kalias(p);
    tic;X=krand(Nevents,KLT)-1;toc
    hist(X,0:50);
    grid
    ax=axis;
    axis([0 max(X) ax(3) ax(4)]);
    disp(['<X>=' num2str(mean(X)) ' +/- ' num2str(std(X))])
    toc
   case 4					% gaussian
    disp(['Extracting ' num2str(Nevents)...
	  ' events from Gaussian distribution']);
    x=linspace(-10,10,200);
    tic;KLTc=krndini('gaussian',x,0,2);toc
    X=krandc(Nevents,KLTc);
   case 5
    disp(['Extracting ' num2str(Nevents)...
	  ' events from Lorentzian distribution']);
    x=linspace(-100,100,400);		% Lorentzian
    tic;KLTc=krndini('lorentz',x,0,20);toc
    X=krandc(Nevents,KLTc);
   case 6
    disp(['Extracting ' num2str(Nevents)...
	  ' events from linear distribution']);
    n=1:100;
    p=n/sum(n);
    tic;KLT=kalias(p);
    X=krand(Nevents,KLT);t1=toc;
    N=histc(X,n);
    tic;exact=lpdf(Nevents,100);t2=toc;
    Nex=histc(exact,n);
    h1=stairs(n,N./n*10100/(2*Nevents));hold on;
    h2=stairs(n,Nex./n*10100/(2*Nevents));hold off;
    axis tight
    set(h1,'Color','g','LineWidth',2);
    set(h2,'Color','r','LineWidth',2);
    title('freq(n)/p(n) ')
    axis([0,100,.9,1.1])
    disp(['Time for alias method:',num2str(t1)])
    disp(['Time for direct method:',num2str(t2)])
    
   otherwise
    check='quit';
    return
  end
end
