function E=cdrand(N,sample,col)
% Statistics of eigenvalues for random stochastic matrices 
% The measure is uniform on the simplex "Sum x_{ij}=1, x_{ij}>0" 
% The rule is to choose each column distributed as exp(-x) and
% then normalizing by taking x_{ij}./sum(x)
% The simple recipe x=rand(N) followed by normalization yields a
% different measure with very similar spectrum

  
  if(nargin<3)
    col='b';
  end
  phi=linspace(0,2*pi,100);
  plot(exp(i*phi),'r','LineWidth',2);
  hold on
  
  xi=-log(rand(N));
  x=xi./(ones(max(size(xi)),1)*sum(xi));
  W=eig(x);
  E=W;
  h=plot(W*sqrt(N),'.','MarkerSize',1,'EraseMode','none');
  axis([-1.1,1.1,-1.1,1.1]);
  axis square
  
  for j=1:sample
    xi=-log(rand(N));
    x=xi./(ones(max(size(xi)),1)*sum(xi));
    W=eig(x);
    set(h,'XData',real(W)*sqrt(N),'YData',imag(W)*sqrt(N))
    drawnow
    E=[E; W];
  end     
  
  E=E(find(abs(E)<.99))*sqrt(N);

  figure
  
  [nr,xr]=hist(real(E),41);
  [ni,xi]=hist(imag(E),41);
  [na,xa]=hist(abs(E),41);
  
  subplot(311)
  stairs(xr-1/41,41*nr/N/sample,col);
  title('Real part')
  subplot(312)
  stairs(xi-1/41,41*ni/N/sample,col);
  title('Imag part')
  subplot(313)
  stairs(xa-1/82,41*na/N/sample,col);
  title('Abs value')
    
% 41* is equivalent to dividing by dn which in our case is 1/41