function ikdemo(Nevents)
% application of Knuth's alias method 
% Source: D.Knuth, The Art of Computer Programming, Vol.2
% See also: Computing in Science and Eng., July issue 1999, paper
% by I.Beichl and F.Sullivan
%
% E.Onofri 2000
% Right to copy for the whole package
% $10.00 - to be accredited to the
% "Physics Promotion Fund", c/o Dipartimento di Fisica, Universit di Parma
% Parco Area delle Scienze, 43100 Parma

if nargin<1,
  Nevents=1e5 
end

close all
lorenz=inline('((x-x0).^2+sigma^2).^(-1)','x','x0','sigma');
gauss=inline('exp(-(x-x0).^2/(2*sigma))','x','x0','sigma');

check='go';

while(check=='go'),
	selection = {'dice throw','diffraction pattern','poisson', ...
			'gaussian','lorentzian','quit'};
	choice    = menu('Start',selection);
	
	switch choice		
	case 1					% Knuth's example
		result=zeros(3,1);
		disp('Alice, Bruno and Charles are drawing dice for 100 times')
		players={'Alice';'Bruno';'Charles'};
		p=[1 2 3 4 5 6 5 4 3 2 1];		
		KLT=kalias(p)
		krand(1,KLT,'init');
		for j=1:100,
			X=krand(3,KLT);
			s=max(X);
			ind=find(X==s);
			result(ind)=result(ind)+1/length(ind);
		end
		score=max(result);
		win=find(result==score);
		disp(['The winner is ' players{win} ' by ' num2str(score) ' points ']);
		bar(result);
	case 2			% fringe
		disp('Running <diffraction pattern demo>');
		disp([num2str(Nevents)...
				' scintillation counts from low energy electron beam']);
		x=linspace(0,1,100);
		KLTc=krndini('fringe',x,4,0);
		tic;X=krandc(Nevents,KLTc);toc
		figure
		plot(X,rand(size(X)),'b.','Markersize',1)
		legend('Diffraction pattern')
	case 3					% Poisson
		disp(['Extracting ' num2str(Nevents) ' events from Poisson distribution']);
		p=poisson(50,10);
		KLT=kalias(p);
		tic;X=krand(Nevents,KLT)-1;toc
		hist(X,0:50);
		grid
		ax=axis;
		axis([0 max(X) ax(3) ax(4)]);
		disp(['<X>=' num2str(mean(X)) ' +/- ' num2str(std(X))])
	case 4					% gaussian
		disp(['Extracting ' num2str(Nevents)...
				' events from Gaussian distribution']);
		x=linspace(-10,10,200);
		tic;KLTc=krndini(gauss,x,0,2);toc
		X=krandc(Nevents,KLTc);
	case 5
		disp(['Extracting ' num2str(Nevents)...
				' events from Lorentzian distribution']);
		x=linspace(-100,100,400);		% Lorentzian
		tic;KLTc=krndini(lorenz,x,0,20);toc
		X=krandc(Nevents,KLTc);
	otherwise
		check='quit';
		return
	end
end

%--------------