function X=krand(NRAND,KLT,seed) 
% Returns NRAND random integers in the interval 1:n (n = length(Y))
% distributed according to the probability vector p(1) p(2) ... p(n)
% which is encoded in the lookup tables KLT.Y and KLT.P prepared through
% Knuth's alias method: KLT=kalias(p); 
% Usage:
% X=krand(N,KLT,'init') % returns N random elements from the
% set (1...length(Y)) after initializing the random number generator,
% seed is chosen as 'sum(clock)*100
%
% X=krand(N,KLT,number) % returns N random elements from the
% set (1...length(Y)) after initializing the random number generator,
% using seed=number
%
% X=krand(1e6,KLT) % returns one million random elements from the
% set (1...length(Y)) without resetting rng's seed
%
% X=krand  (returns a demo, Poisson distribution)
% 
% n=0:50;
% mu=10;
% p=mu.^n.*exp(-mu)./gamma(n+1);
% KLT=kalias(p,1);
% X=krand(1e6,KLT,'init');
% hist(X,n)
%
% Requires: kalias.m
% Source: D.Knuth, The Art of Computer Programming, Vol.2
% See also: Computing in Science and Eng., July issue 1999, paper
% by I.Beichl and F.Sullivan
% E.Onofri, 1998-2002

P=KLT.P;
Y=KLT.Y;
if nargin==4,
  if isstr(seed),
    rand('state',sum(clock)*100);
  else
    rand('state',seed);
  end
end
if nargin>0,
N=length(P);
r=N*rand(1,NRAND);  
X=ceil(r);
v=X-r;
%v=rand(1,NRAND); % if not use this line instead
ind=find(v>P(X));
X(ind)=Y(X(ind)); 
else
   disp('Krand: demo of Knuth''s alias method')
   disp('Generating 10^6 events from Poisson distribution')
   disp('with mean=10')
	   
	n=0:50;
	mu=10;
	p=mu.^n.*exp(-mu)./gamma(n+1);
	[Y,P]=kalias(p,1);
	X=krand(1e6,Y,P,'init')-1;  %indices start at 1 in Matlab! 
   hist(X,n);
   grid
   ax=axis;
   axis([0 max(X) ax(3) ax(4)]);
   disp(['<X>=' num2str(mean(X)) ' +/- ' num2str(std(X))])
end
