function X = krandm(NRAND, KLT, copies) 
% matrix 'krand' 
% Returns a matrix copies x NRAND of random integers in the interval
% 1:n (n = length(Y)) distributed according to the probability vector
%  p(1) p(2) ... p(n)
% which is encoded in the lookup tables KLT.Y and KLT.P prepared through
% Knuth's alias method: KLT=kalias(p); 
% Usage:
% X = krandm(N,KLT,copies) % returns an array Nxcopies random elements
% from the set (1...length(Y)) %
% Requires: kalias.m
% Source: D.Knuth, The Art of Computer Programming, Vol.2
% See also: Computing in Science and Eng., July issue 1999, paper
% by I.Beichl and F.Sullivan
% E.Onofri, 1998-2002-2019

if nargin<1,
   disp('please provide all input parameters!')
   return
end


P = KLT.P;
Y = KLT.Y;

if nargin<3, copies=1;end

N = length(P);

X = ceil(N*rand(copies, NRAND));  

v = rand(copies, NRAND); 

ind = find(v>P(X));

X(ind) = Y(X(ind));			% choose alias 

