function KLTc = krndini(varargin)
% Initialize lookup table for krandc
% Usage: KLTc = krndini(func,x,[P1,P2,...])
% x is the 1-dim mesh
% 'func' is a user-defined function giving the probability density
% Parameters can be passed to 'func' as P1, P2, ...
% To get a sample of N numbers distributed 
% according to 'func', enter:
% X = krandc(N,KLTc,seed);
% Example:
% Generate 1000 samples from a Lorentzian distribution
% x=linspace(-20,20,200);
% KLTc=krndini('lorentz',x,1,.5)
% X=krandc(1000,KLTc);
% E.ONOFRI, 1998-2000 (onofri@pr.infn.it)

if(length(varargin)<2),
  error('Please supply density and x-values');
end

func=varargin{1};
x=varargin{2};
KLTc.mesh=x;
n=length(x);
xm=x(1:n-1);
xp=x(2:n);

p = feval(func,x,varargin{3:end});

if(min(p)<0),
  error(['p cannot be negative. Check function ' func]);
  return;
end

p1 = .5*(feval(func,xp,varargin{3:end})+feval(func,xm,varargin{3:end}));
p=p/sum(p);
p1=p1/sum(p1);
KLTc.p=p;

KLT=kalias(p1);
KLTc.Y=KLT.Y;
KLTc.P=KLT.P;
