function loop(n1,n2,Nmax, fsave)

% Statistics of eigenvalues for random stochastic matrices 
% The measure is uniform on the simplex "Sum x_{ij}=1, x_{ij}>0" 
% The rule is to choose each column distributed as exp(-x) and
% then normalizing by taking x_{ij}./sum(x)
% The simple recipe x=rand(N) followed by normalization yields a
% different measure with very similar spectrum
% Usage:  loop(n1,n2,Nmax,fsave)

i=1;

if nargin<4
  fsave='loop';
end

if nargin<3
  disp('Usage:  loop(n1,n2,Nmax,fsave)')
  error('Please provide:  n1, n2, Nmax');
end

dim = 2.^(n1:n2);
col = ['b' 'g' 'm' 'k' 'r' 'b' 'g' 'm' 'k' 'r' 'b' 'g' 'm' 'k' 'r' ];

for N=dim,
  
  sample = ceil(Nmax/N)
  
  E=[];
  
  for j=1:sample
    xi=-log(rand(N));
    xi=xi./(ones(max(size(xi)),1)*sum(xi));
    W=eig(xi);
    E=[E; W];
  end     
  
  eval(['E' num2str(N) '=E;']);
  
  %  x=linspace(-1/sqrt(N),1/sqrt(N),length(E));
  subplot(121)
  x=linspace(-1,1,length(E));
  plot(sqrt(N)*sort(imag(E)),x,col(i));%,'LineWidth',2);
  hold on
  j=find(abs(real(E))<.99);
  subplot(122)
  plot(sqrt(N)*sort(real(E(j))),x(j),col(i));
  hold on
  i=i+1;
  drawnow
end
n=(x.*sqrt(1-x.^2)+asin(x))*2/pi;
plot(x,n,'k-.');%,'LineWidth',2)
grid
subplot(121)
grid
plot(x,n,'k-.');%,'LineWidth',2)
eval(['save ' fsave]);
