function L=lyapounov(N,sw,n,p,delta);
% liapounov.m: check Liapounov condition on binomial random var.s
% Usage:
%   L=liapounov(N,sw,n,p,delta)
%   sw = number of samples used to compute the average binomial
%   random var.
%   n,p - parameters of binomial distribution
%   N = number of binomial r.v. experiments
%   delta>0 is usually equl to 1

if nargin<5, delta=1; end
if nargin<4, p=1/2; end
if nargin<3, n=5; end
if nargin<2, sw=1000; end
if nargin<1, N=100; end


prob = binomial(n,p);

KLT = kalias(prob);                     % apply the alias method 

L = [];

for j = 1:N,
   X = Krand(sw,KLT)-1;                   % generate sw samples out
                                          % of binomial distribution
   L = [L; ...
        mean(abs(X-n*p).^(2+delta))];
end

L = sum(L)/(N*n*p*(1-p))^((2+delta)/2);   % Lyapounov test: L->0 as N->infty
