function [W,P]=polya(Nsample,n,s,w1,b1)
% Distribuzione di Polya
% ad ogni estrazione alla palla estratta se ne aggiungono $s$ dello
% stesso colore. Inizialmente w1 palle bianche e b1 nere.
% Si fanno n estrazioni
% Si ripete l'esperimento 'Nsample' volte. 
% Wtot raccoglie le estrazioni di palle bianche in ogni esperimento
% e il programma ne costrisce l'istogramma.
% P contiene la distribuzione di Polya corrispondente ai parametri assegnati
% Ref.: Fisz, "Probability Theory and Mathematical Statistics", J.Wiley.
%
% Usage:  [Wtot,P]=polya(Nsample,n,s,w1,b1)
    
    if nargin<2
        n=10
    end

    if nargin<1
        Nsample=1000
    end
    
    if nargin<4
        w1 = 5
        b1 = 5
    end
    if nargin<5,
        b1 = w1
    end
    if nargin<3
        s = 0
    end

    Wtot=[];
    
    for j=1:Nsample
        N=w1+b1;
        p=w1/N;
        W=0;
        w=w1;
        b=b1;
        for k=1:n
            if(rand<p)
                W=W+1;
                w=w+s;
            else
                b=b+s;
            end
            N=w+b;
            p=w/N;
        end
        Wtot=[Wtot,W];
    end
    
    % Polya distribution
    
    N=w1+b1;
    p=w1/N;
    q=b1/N;
    a=s/N;
    nrm=prod(1+a*(0:n-1));
    for  k=1:n
        K=a*(0:k-1);
        H=a*(0:n-k-1);
        P(k)=nchoosek(n,k)*prod(p+K)*prod(q+H)/nrm;
    end
    P(1)=1-sum(P);

    % plot results
    W = Wtot;
    [Nw,x]= hist(W,0:n);
    Nw=Nw/sum(Nw);
    figure
    stairs(x,Nw,'LineWidth',2);
    hold on
    stairs(P,'r','LineWidth',2)
    
    disp(['mean=',num2str(mean(W)),' (ex=',num2str(n*p),')'])
    disp(['std=',num2str(std(W)),' (ex=',num2str(sqrt(n*p*(1-p)*(1+n* ...
                                                      a)/(1+a))),')'])
    disp(['skew=',num2str(mean((W-mean(W)).^3)/std(W)^3)])
    
    grid
    axis([0,n,0,max(max(P),max(Nw))])
    title('drawing from a Polya distribution')
    legend('exp','polya')
