function [Y,P] = alias(p,debug)
% Build lookup tables for Knuth's alias method
% given probabilities  p1,p2,...,p_N 
% Usage: [Y,P] = knuth(p[,debug])
% p=(p1,p2,...,p_N)
% in return:
% Y contains the 'aliases' lookup table and
% P  contains the branching probabilities
% If called with 'debug' option, tables are checked for consistency
% required by: krand.m 
% E.Onofri, 1998-2000

[N,M]=size(p);

% Verify input

if N>1 & M>1,
  error('please provide a one-column-vector as input');
end

if N<M,
	p=p';
	N=M;
end

find(p<0);
if(~isempty(ans)),
  error('p MUST be positive')
end

p=p/sum(p); %normalize p 
% OK, p appears to be consistent!

[L,ind]=sort(p);
L=[L ind];
n=N;

while n>1,
  P(L(1,2))=N*L(1,1);
  Y(L(1,2))=L(n,2);
  L(n,1)=L(n,1)-(1/N-L(1,1));
  L=L(2:n,:);
  [tmp,ind]=sort(L(:,1));
  L=L(ind,:);
  n=n-1;
end % now n=1, ... last entry

P(L(1,2))=N*L(1,1);
Y(L(1,2))=L(n,2);

q=[];

if nargin>1, % debug 
  for j=1:N,
    P1=1-P(find(Y==j));
    q=[q; (P(j)+sum(P1))/N];
  end
  error=	norm(q-p)/N;		
  if(error<=2*eps),
    disp('Tables are correct to machine precision');
  else
    error('tables are inconsistent');
  end
end


